# The COPYRIGHT file at the top level of this repository contains the full
# copyright notices and license terms.
from trytond.pool import Pool
from . import batch
from . import journal
from . import move
from . import party

__all__ = ['register']


def register():
    Pool.register(
        journal.Journal,
        journal.BatchJournal,
        batch.Batch,
        batch.BatchLine,
        batch.CancelBatchLinesAskSure,
        batch.CancelBatchLinesAskProblem,
        batch.OpenBatchJournalAsk,
        move.Move,
        move.MoveLine,
        module='account_batch', type_='model')
    Pool.register(
        batch.CancelBatchLines,
        batch.OpenBatchJournal,
        batch.PostBatchLines,
        party.PartyReplace,
        module='account_batch', type_='wizard')



# The COPYRIGHT file at the top level of this repository contains
# the full copyright notices and license terms.
import unittest
import doctest
from trytond.tests.test_tryton import ModuleTestCase
from trytond.tests.test_tryton import suite as test_suite
from trytond.tests.test_tryton import doctest_setup, doctest_teardown, \
    doctest_checker


class AccountBatchTestCase(ModuleTestCase):
    'Test Account Batch module'
    module = 'account_batch'


def suite():
    suite = test_suite()
    suite.addTests(doctest.DocFileSuite(
            'scenario_account_batch.rst',
            setUp=doctest_setup, tearDown=doctest_teardown, encoding='utf-8',
            checker=doctest_checker,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
    # Run this test after the scenario, because the scenario needs a fresh
    # database. SQLite doesn't seem to properly rollback the transaction.
    # s. a. https://bugs.tryton.org/issue9133
    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
            AccountBatchTestCase))
    return suite



# The COPYRIGHT file at the top level of this repository contains the full
# copyright notices and license terms.
from sql import Table

from trytond import backend
from trytond.model import ModelView, ModelSQL, DeactivableMixin, fields
from trytond.transaction import Transaction
from trytond.pool import Pool, PoolMeta
from trytond.pyson import Eval


class BatchJournal(DeactivableMixin, ModelSQL, ModelView):
    'Batch Journal'
    __name__ = 'account.batch.journal'

    name = fields.Char('Name', required=True)
    account_journal = fields.Many2One('account.journal', 'Account Journal',
            required=True)
    currency = fields.Many2One('currency.currency', 'Currency', required=True)
    company = fields.Many2One('company.company', 'Company', required=True)
    account = fields.Many2One('account.account', "Account",
        domain=[
            ('type', '!=', None),
            ('closed', '!=', True),
            ('company', '=', Eval('company')),
            ],
        depends=['company'])

    @classmethod
    def __register__(cls, module_name):
        TableHandler = backend.get('TableHandler')
        cursor = Transaction().connection.cursor()
        table = TableHandler(cls, module_name)
        sql_table = cls.__table__()
        journal_account = Table('account_journal_account')

        created_account = not table.column_exist('account')

        super(BatchJournal, cls).__register__(module_name)

        # Migration from 4.8: new account field
        if created_account and table.table_exist('account_journal_account'):
            value = journal_account.select(journal_account.credit_account,
                where=((journal_account.journal == sql_table.journal) &
                    (journal_account.credit_account ==
                        journal_account.debit_account)))
            # Don't use UPDATE FROM because SQLite does not support it.
            cursor.execute(*sql_table.update([sql_table.account], [value]))

    @staticmethod
    def default_currency():
        Company = Pool().get('company.company')
        if Transaction().context.get('company'):
            company = Company(Transaction().context['company'])
            return company.currency.id

    @staticmethod
    def default_company():
        return Transaction().context.get('company')


class Journal(metaclass=PoolMeta):
    __name__ = 'account.journal'

    @classmethod
    def __setup__(cls):
        super(Journal, cls).__setup__()
        cls.type.selection.append(('bank', 'Bank'))
